@@ -1,35 +1,22 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """QtWebKit specific part of the web element API."""
 
-import typing
+from typing import cast, TYPE_CHECKING, Iterator, List, Optional, Set
 
-from PyQt5.QtCore import QRect, Qt
-from PyQt5.QtWebKit import QWebElement, QWebSettings
-from PyQt5.QtWebKitWidgets import QWebFrame
+from qutebrowser.qt.core import QRect, Qt
+# pylint: disable=no-name-in-module
+from qutebrowser.qt.webkit import QWebElement, QWebSettings
+from qutebrowser.qt.webkitwidgets import QWebFrame
+# pylint: enable=no-name-in-module
 
 from qutebrowser.config import config
 from qutebrowser.utils import log, utils, javascript, usertypes
 from qutebrowser.browser import webelem
 
-if typing.TYPE_CHECKING:
+if TYPE_CHECKING:
     from qutebrowser.browser.webkit import webkittab
 
 
@@ -42,6 +29,8 @@ class WebKitElement(webelem.AbstractWebElement):
 
     """A wrapper around a QWebElement."""
 
+    _tab: 'webkittab.WebKitTab'
+
     def __init__(self, elem: QWebElement, tab: 'webkittab.WebKitTab') -> None:
         super().__init__(tab)
         if isinstance(elem, self.__class__):
@@ -80,7 +69,7 @@ class WebKitElement(webelem.AbstractWebElement):
         self._check_vanished()
         return self._elem.hasAttribute(key)
 
-    def __iter__(self) -> typing.Iterator[str]:
+    def __iter__(self) -> Iterator[str]:
         self._check_vanished()
         yield from self._elem.attributeNames()
 
@@ -101,7 +90,7 @@ class WebKitElement(webelem.AbstractWebElement):
         self._check_vanished()
         return self._elem.geometry()
 
-    def classes(self) -> typing.Set[str]:
+    def classes(self) -> Set[str]:
         self._check_vanished()
         return set(self._elem.classes())
 
@@ -174,21 +163,16 @@ class WebKitElement(webelem.AbstractWebElement):
             this.dispatchEvent(event);
         """.format(javascript.to_js(text)))
 
-    def _parent(self) -> typing.Optional['WebKitElement']:
+    def _parent(self) -> Optional['WebKitElement']:
         """Get the parent element of this element."""
         self._check_vanished()
-        elem = typing.cast(typing.Optional[QWebElement],
-                           self._elem.parent())
+        elem = cast(Optional[QWebElement], self._elem.parent())
         if elem is None or elem.isNull():
             return None
 
-        if typing.TYPE_CHECKING:
-            # pylint: disable=used-before-assignment
-            assert isinstance(self._tab, webkittab.WebKitTab)
-
         return WebKitElement(elem, tab=self._tab)
 
-    def _rect_on_view_js(self) -> typing.Optional[QRect]:
+    def _rect_on_view_js(self) -> Optional[QRect]:
         """Javascript implementation for rect_on_view."""
         # FIXME:qtwebengine maybe we can reuse this?
         rects = self._elem.evaluateJavaScript("this.getClientRects()")
@@ -218,7 +202,7 @@ class WebKitElement(webelem.AbstractWebElement):
                 rect = QRect(int(rect["left"]), int(rect["top"]),
                              int(width), int(height))
 
-                frame = typing.cast(typing.Optional[QWebFrame], self._elem.webFrame())
+                frame = cast(Optional[QWebFrame], self._elem.webFrame())
                 while frame is not None:
                     # Translate to parent frames' position (scroll position
                     # is taken care of inside getClientRects)
@@ -229,8 +213,7 @@ class WebKitElement(webelem.AbstractWebElement):
 
         return None
 
-    def _rect_on_view_python(self,
-                             elem_geometry: typing.Optional[QRect]) -> QRect:
+    def _rect_on_view_python(self, elem_geometry: Optional[QRect]) -> QRect:
         """Python implementation for rect_on_view."""
         if elem_geometry is None:
             geometry = self._elem.geometry()
@@ -238,11 +221,11 @@ class WebKitElement(webelem.AbstractWebElement):
             geometry = elem_geometry
         rect = QRect(geometry)
 
-        frame = typing.cast(typing.Optional[QWebFrame], self._elem.webFrame())
+        frame = cast(Optional[QWebFrame], self._elem.webFrame())
         while frame is not None:
             rect.translate(frame.geometry().topLeft())
             rect.translate(frame.scrollPosition() * -1)
-            frame = typing.cast(typing.Optional[QWebFrame], frame.parentFrame())
+            frame = cast(Optional[QWebFrame], frame.parentFrame())
 
         return rect
 
@@ -280,7 +263,7 @@ class WebKitElement(webelem.AbstractWebElement):
     def _is_hidden_css(self) -> bool:
         """Check if the given element is hidden via CSS."""
         attr_values = {
-            attr: self._elem.styleProperty(attr, QWebElement.ComputedStyle)
+            attr: self._elem.styleProperty(attr, QWebElement.StyleResolveStrategy.ComputedStyle)
             for attr in ['visibility', 'display', 'opacity']
         }
         invisible = attr_values['visibility'] == 'hidden'
@@ -336,7 +319,7 @@ class WebKitElement(webelem.AbstractWebElement):
         return all([visible_on_screen, visible_in_frame])
 
     def remove_blank_target(self) -> None:
-        elem = self  # type: typing.Optional[WebKitElement]
+        elem: Optional[WebKitElement] = self
         for _ in range(5):
             if elem is None:
                 break
@@ -366,7 +349,7 @@ class WebKitElement(webelem.AbstractWebElement):
 
     def _click_js(self, click_target: usertypes.ClickTarget) -> None:
         settings = QWebSettings.globalSettings()
-        attribute = QWebSettings.JavascriptCanOpenWindows
+        attribute = QWebSettings.WebAttribute.JavascriptCanOpenWindows
         could_open_windows = settings.testAttribute(attribute)
         settings.setAttribute(attribute, True)
         ok = self._elem.evaluateJavaScript('this.click(); true;')
@@ -376,15 +359,15 @@ class WebKitElement(webelem.AbstractWebElement):
             self._click_fake_event(click_target)
 
     def _click_fake_event(self, click_target: usertypes.ClickTarget,
-                          button: Qt.MouseButton = Qt.LeftButton) -> None:
+                          button: Qt.MouseButton = Qt.MouseButton.LeftButton) -> None:
         self._tab.data.override_target = click_target
         super()._click_fake_event(click_target)
 
 
-def get_child_frames(startframe: QWebFrame) -> typing.List[QWebFrame]:
+def get_child_frames(startframe: QWebFrame) -> List[QWebFrame]:
     """Get all children recursively of a given QWebFrame.
 
-    Loosely based on http://blog.nextgenetics.net/?e=64
+    Loosely based on https://blog.nextgenetics.net/?e=64
 
     Args:
         startframe: The QWebFrame to start with.
@@ -395,7 +378,7 @@ def get_child_frames(startframe: QWebFrame) -> typing.List[QWebFrame]:
     results = []
     frames = [startframe]
     while frames:
-        new_frames = []  # type: typing.List[QWebFrame]
+        new_frames: List[QWebFrame] = []
         for frame in frames:
             results.append(frame)
             new_frames += frame.childFrames()
