@@ -1,26 +1,12 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Exceptions related to config parsing."""
 
-import typing
-import attr
+import difflib
+import dataclasses
+from typing import Any, Mapping, Optional, Sequence, Union, List
 
 from qutebrowser.utils import usertypes, log
 
@@ -46,7 +32,7 @@ class BackendError(Error):
     def __init__(
             self, name: str,
             backend: usertypes.Backend,
-            raw_backends: typing.Optional[typing.Mapping[str, bool]]
+            raw_backends: Optional[Mapping[str, bool]]
     ) -> None:
         if raw_backends is None or not raw_backends[backend.name]:
             msg = ("The {} setting is not available with the {} backend!"
@@ -76,8 +62,7 @@ class ValidationError(Error):
         msg: Additional error message.
     """
 
-    def __init__(self, value: typing.Any,
-                 msg: typing.Union[str, Exception]) -> None:
+    def __init__(self, value: Any, msg: Union[str, Exception]) -> None:
         super().__init__("Invalid value '{}' - {}".format(value, msg))
         self.option = None
 
@@ -92,6 +77,7 @@ class NoOptionError(Error):
     """Raised when an option was not found."""
 
     def __init__(self, option: str, *,
+                 all_names: List[str] = None,
                  deleted: bool = False,
                  renamed: str = None) -> None:
         if deleted:
@@ -99,6 +85,12 @@ class NoOptionError(Error):
             suffix = ' (this option was removed from qutebrowser)'
         elif renamed is not None:
             suffix = ' (this option was renamed to {!r})'.format(renamed)
+        elif all_names:
+            matches = difflib.get_close_matches(option, all_names, n=1)
+            if matches:
+                suffix = f' (did you mean {matches[0]!r}?)'
+            else:
+                suffix = ''
         else:
             suffix = ''
 
@@ -106,7 +98,7 @@ class NoOptionError(Error):
         self.option = option
 
 
-@attr.s
+@dataclasses.dataclass
 class ConfigErrorDesc:
 
     """A description of an error happening while reading the config.
@@ -117,9 +109,9 @@ class ConfigErrorDesc:
         traceback: The formatted traceback of the exception.
     """
 
-    text = attr.ib()  # type: str
-    exception = attr.ib()  # type: typing.Union[str, Exception]
-    traceback = attr.ib(None)  # type: str
+    text: str
+    exception: Union[str, Exception]
+    traceback: Optional[str] = None
 
     def __str__(self) -> str:
         if self.traceback:
@@ -141,7 +133,7 @@ class ConfigFileErrors(Error):
 
     def __init__(self,
                  basename: str,
-                 errors: typing.Sequence[ConfigErrorDesc], *,
+                 errors: Sequence[ConfigErrorDesc], *,
                  fatal: bool = False) -> None:
         super().__init__("Errors occurred while reading {}:\n{}".format(
             basename, '\n'.join('  {}'.format(e) for e in errors)))
@@ -150,8 +142,7 @@ class ConfigFileErrors(Error):
         self.fatal = fatal
         for err in errors:
             if err.traceback:
-                log.config.debug("Config error stack:")
-                log.config.debug(err.traceback)
+                log.config.info(err.traceback)
 
     def to_html(self) -> str:
         """Get the error texts as a HTML snippet."""
