@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Utils regarding URL handling."""
 
@@ -26,10 +11,11 @@ import ipaddress
 import posixpath
 import urllib.parse
 import mimetypes
-from typing import Optional, Tuple, Union
+from typing import Optional, Tuple, Union, Iterable, cast
 
-from PyQt5.QtCore import QUrl
-from PyQt5.QtNetwork import QHostInfo, QHostAddress, QNetworkProxy
+from qutebrowser.qt import machinery
+from qutebrowser.qt.core import QUrl
+from qutebrowser.qt.network import QHostInfo, QHostAddress, QNetworkProxy
 
 from qutebrowser.api import cmdutils
 from qutebrowser.config import config
@@ -41,6 +27,58 @@ from qutebrowser.browser.network import pac
 # https://github.com/qutebrowser/qutebrowser/issues/108
 
 
+if machinery.IS_QT6:
+    UrlFlagsType = Union[QUrl.UrlFormattingOption, QUrl.ComponentFormattingOption]
+
+    class FormatOption:
+        """Simple wrapper around Qt enums to fix typing problems on Qt 5."""
+
+        ENCODED = QUrl.ComponentFormattingOption.FullyEncoded
+        ENCODE_UNICODE = QUrl.ComponentFormattingOption.EncodeUnicode
+        DECODE_RESERVED = QUrl.ComponentFormattingOption.DecodeReserved
+
+        REMOVE_SCHEME = QUrl.UrlFormattingOption.RemoveScheme
+        REMOVE_PASSWORD = QUrl.UrlFormattingOption.RemovePassword
+        REMOVE_QUERY = QUrl.UrlFormattingOption.RemoveQuery
+else:
+    UrlFlagsType = Union[
+        QUrl.FormattingOptions,
+        QUrl.UrlFormattingOption,
+        QUrl.ComponentFormattingOption,
+        QUrl.ComponentFormattingOptions,
+    ]
+
+    class _QtFormattingOptions(QUrl.FormattingOptions):
+        """WORKAROUND for invalid stubs.
+
+        Teach mypy that | works for QUrl.FormattingOptions.
+        """
+
+        def __or__(self, f: UrlFlagsType) -> '_QtFormattingOptions':
+            return super() | f  # type: ignore[operator,return-value]
+
+    class FormatOption:
+        """WORKAROUND for invalid stubs.
+
+        Pretend that all ComponentFormattingOption values are also valid
+        QUrl.FormattingOptions values, i.e. can be passed to QUrl.toString().
+        """
+
+        ENCODED = cast(
+            _QtFormattingOptions, QUrl.ComponentFormattingOption.FullyEncoded)
+        ENCODE_UNICODE = cast(
+            _QtFormattingOptions, QUrl.ComponentFormattingOption.EncodeUnicode)
+        DECODE_RESERVED = cast(
+            _QtFormattingOptions, QUrl.ComponentFormattingOption.DecodeReserved)
+
+        REMOVE_SCHEME = cast(
+            _QtFormattingOptions, QUrl.UrlFormattingOption.RemoveScheme)
+        REMOVE_PASSWORD = cast(
+            _QtFormattingOptions, QUrl.UrlFormattingOption.RemovePassword)
+        REMOVE_QUERY = cast(
+            _QtFormattingOptions, QUrl.UrlFormattingOption.RemoveQuery)
+
+
 # URL schemes supported by QtWebEngine
 WEBENGINE_SCHEMES = [
     'about',
@@ -95,6 +133,7 @@ def _parse_search_term(s: str) -> Tuple[Optional[str], Optional[str]]:
             engine = None
             term = s
     else:
+        # pylint: disable=else-if-used
         if config.val.url.open_base_url and s in config.val.url.searchengines:
             engine = s
             term = None
@@ -130,9 +169,9 @@ def _get_search_url(txt: str) -> QUrl:
         url = QUrl.fromUserInput(evaluated)
     else:
         url = QUrl.fromUserInput(config.val.url.searchengines[engine])
-        url.setPath(None)  # type: ignore[arg-type]
-        url.setFragment(None)  # type: ignore[arg-type]
-        url.setQuery(None)  # type: ignore[call-overload]
+        url.setPath(qtutils.QT_NONE)
+        url.setFragment(qtutils.QT_NONE)
+        url.setQuery(qtutils.QT_NONE)
     qtutils.ensure_valid(url)
     return url
 
@@ -328,6 +367,7 @@ def invalid_url_error(url: QUrl, action: str) -> None:
     """Display an error message for a URL.
 
     Args:
+        url: The URL to display a message for.
         action: The action which was interrupted by the error.
     """
     if url.isValid():
@@ -491,8 +531,22 @@ def same_domain(url1: QUrl, url2: QUrl) -> bool:
     if url1.port() != url2.port():
         return False
 
-    suffix1 = url1.topLevelDomain()
-    suffix2 = url2.topLevelDomain()
+    # QUrl.topLevelDomain() got removed in Qt 6:
+    # https://bugreports.qt.io/browse/QTBUG-80308
+    #
+    # However, we should never land here if we are on Qt 6:
+    #
+    # On QtWebEngine, we don't have a QNetworkAccessManager attached to a tab
+    # (all tab-specific downloads happen via the QtWebEngine network stack).
+    # Thus, ensure_valid(url2) above will raise InvalidUrlError, which is
+    # handled in NetworkManager.
+    #
+    # There are no other callers of same_domain, and url2 will only be ever valid when
+    # we use a NetworkManager from QtWebKit. However, QtWebKit is Qt 5 only.
+    assert machinery.IS_QT5, machinery.INFO
+
+    suffix1 = url1.topLevelDomain()  # type: ignore[attr-defined,unused-ignore]
+    suffix2 = url2.topLevelDomain()  # type: ignore[attr-defined,unused-ignore]
     if not suffix1:
         return url1.host() == url2.host()
 
@@ -520,7 +574,7 @@ def file_url(path: str) -> str:
         path: The absolute path to the local file
     """
     url = QUrl.fromLocalFile(path)
-    return url.toString(QUrl.FullyEncoded)  # type: ignore[arg-type]
+    return url.toString(FormatOption.ENCODED)
 
 
 def data_url(mimetype: str, data: bytes) -> QUrl:
@@ -542,11 +596,11 @@ def safe_display_string(qurl: QUrl) -> str:
     """
     ensure_valid(qurl)
 
-    host = qurl.host(QUrl.FullyEncoded)
+    host = qurl.host(QUrl.ComponentFormattingOption.FullyEncoded)
     assert '..' not in host, qurl  # https://bugreports.qt.io/browse/QTBUG-60364
 
     for part in host.split('.'):
-        url_host = qurl.host(QUrl.FullyDecoded)
+        url_host = qurl.host(QUrl.ComponentFormattingOption.FullyDecoded)
         if part.startswith('xn--') and host != url_host:
             return '({}) {}'.format(host, qurl.toDisplayString())
 
@@ -579,10 +633,10 @@ def proxy_from_url(url: QUrl) -> Union[QNetworkProxy, pac.PACFetcher]:
         return fetcher
 
     types = {
-        'http': QNetworkProxy.HttpProxy,
-        'socks': QNetworkProxy.Socks5Proxy,
-        'socks5': QNetworkProxy.Socks5Proxy,
-        'direct': QNetworkProxy.NoProxy,
+        'http': QNetworkProxy.ProxyType.HttpProxy,
+        'socks': QNetworkProxy.ProxyType.Socks5Proxy,
+        'socks5': QNetworkProxy.ProxyType.Socks5Proxy,
+        'direct': QNetworkProxy.ProxyType.NoProxy,
     }
     if scheme not in types:
         raise InvalidProxyTypeError(scheme)
@@ -611,7 +665,7 @@ def parse_javascript_url(url: QUrl) -> str:
         raise Error("URL contains unexpected components: {}"
                     .format(url.authority()))
 
-    urlstr = url.toString(QUrl.FullyEncoded)  # type: ignore[arg-type]
+    urlstr = url.toString(FormatOption.ENCODED)
     urlstr = urllib.parse.unquote(urlstr)
 
     code = urlstr[len('javascript:'):]
@@ -619,3 +673,12 @@ def parse_javascript_url(url: QUrl) -> str:
         raise Error("Resulted in empty JavaScript code")
 
     return code
+
+
+def widened_hostnames(hostname: str) -> Iterable[str]:
+    """A generator for widening string hostnames.
+
+    Ex: a.c.foo -> [a.c.foo, c.foo, foo]"""
+    while hostname:
+        yield hostname
+        hostname = hostname.partition(".")[-1]
